<?php

/**
 * @file
 * Strategy with access behaviour
 *
 * @usage
 * $context = new HeartbeatMessageBuilder(new PublicBehaviour());
 * $context->execute();
 *
 * $context = new HeartbeatMessageBuilder(new PrivateBehaviour());
 * $context->execute();
 *
 * $context = new HeartbeatMessageBuilder(new ConnectedBehaviour());
 * $context->execute();
 *
 */

/**
 * Class HeartbeatMessageBuilder
 * Message builder that fetches and customizes a
 * users activity on the site.
 * The builder takes a heartbeataccess state object to
 * handle the request to fetch the messages.
 *
 */
class HeartbeatMessageBuilder {

  private $_heartbeatState = null;
  private $_hasErrors = FALSE;
  private $_errors = array();

  public function __construct(HeartbeatAccess $heartbeatState) {

  	if(!$heartbeatState->hasErrors()) {
  		$this->_heartbeatState = $heartbeatState;
  	}
  	else {
  		$this->setError($heartbeatState->getErrors());
  	}
  }

  public function hasErrors() {
    return $this->_hasErrors;
  }

  protected function setError($errors) {
    $this->_errors = $errors;
    $this->_hasErrors = TRUE;
  }

  public function execute() {

    $heartbeat = $this->_heartbeatState->fetchMessages();

    if(!empty($heartbeat->raw_messages)) {

      $this->checkAccess($heartbeat);

    	return $this->group($heartbeat);
    }

    return array();
  }

  /**
   * Function to check access on messages
   * This behaviour is set by a heartbeat message configuration
   * to overrule the chosen display access type
   */
  private function checkAccess(HeartbeatParser $heartbeat) {

  	foreach ($heartbeat->raw_messages as $key => $message) {
  		// unset the private messages when showing a display to others
  		if($message->perms == HEARTBEAT_PRIVATE && !($this->_heartbeatState instanceOf PrivateHeartbeat)) {
  			unset($heartbeat->raw_messages[$key]);
  		}
  	}
  }

  /**
   * Function to check if more/older messages can be loaded
   *
   * @return Boolean has more messages
   */
  public function hasMoreMessages() {
  	return variable_get($access_type . '_page_show_pager', TRUE) && $this->_heartbeatState->getInfo()->pager;
  }

  /**
   * @see class HeartbeatParser
   */
  public function group(HeartbeatParser $heartbeat) {

	  $heartbeat->set_timespan_gap(variable_get('heartbeat_activity_grouping_seconds',720));
	  $heartbeat->build_sets($heartbeat->raw_messages);
	  $heartbeat->merge_sets();
	  $messages = $heartbeat->get_messages();
	  //  $messages = $heartbeat->remove_broken_messages();

    return $messages;
  }
}